@@ -8,7 +8,7 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdhandler.h 6156 2012-01-01 21:02:08Z rousseau $
+ * $Id: ifdhandler.h 6357 2012-06-25 13:53:30Z rousseau $
  */
 
 /**
@@ -19,7 +19,7 @@
 The routines specified hereafter will allow you to write an IFD handler
 for the PC/SC Lite resource manager. Please use the complement
 developer's kit complete with headers and Makefile at:
-http://www.musclecard.com/drivers.html.
+http://www.musclecard.com/drivers.html
 
 This gives a common API for communication to most readers in a
 homogeneous fashion. This document assumes that the driver developer is
@@ -414,16 +414,15 @@ port used by each reader.
 
 - libusb
 
-  For USB drivers using libusb http://libusb.sourceforge.net/ for USB
+  For USB drivers using libusb-1.0 http://libusb.sourceforge.net/ for USB
   abstraction the @p DeviceName the string may be generated by:
 
   @code
-  printf("usb:%04x/%04x:libusb:%s:%s",
-    idVendor, idProduct,
-    bus->dirname, dev->filename)
+  printf("usb:%04x/%04x:libusb-1.0:%d:%d:%d",
+    idVendor, idProduct, bus_number, device_address, interface)
   @endcode
 
-  So it is something like: <tt>usb:08e6/3437:libusb:001:042</tt> under
+  So it is something like: <tt>usb:08e6/3437:libusb-1.0:7:99:0</tt> under
   GNU/Linux.
 
 - libudev
