@@ -6,7 +6,7 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: debuglog.c 6028 2011-10-11 14:18:43Z rousseau $
+ * $Id: debuglog.c 6229 2012-02-21 08:49:21Z rousseau $
  */
 
 /**
@@ -202,6 +202,10 @@ void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
 		|| (DEBUGLOG_NO_DEBUG == LogMsgType))
 		return;
 
+	/* len is an error value? */
+	if (len < 0)
+		return;
+
 	log_xxd_always(priority, msg, buffer, len);
 } /* log_xxd */
 
