@@ -3,10 +3,10 @@
  *
  * Copyright (C) 2000-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.c 5415 2010-11-23 08:30:14Z rousseau $
+ * $Id: eventhandler.c 5859 2011-07-09 11:28:20Z rousseau $
  */
 
 /**
@@ -26,7 +26,6 @@
 
 #include "misc.h"
 #include "pcscd.h"
-#include "ifdhandler.h"
 #include "debuglog.h"
 #include "readerfactory.h"
 #include "eventhandler.h"
@@ -229,14 +228,12 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 	 * Zero out everything
 	 */
 	dwStatus = 0;
-	readerSharing = 0;
-	dwCurrentState = 0;
 
 	readerName = rContext->readerState->readerName;
 
 	rv = IFDStatusICC(rContext, &dwStatus);
 
-	if (dwStatus & SCARD_PRESENT)
+	if ((SCARD_S_SUCCESS == rv) && (dwStatus & SCARD_PRESENT))
 	{
 #ifdef DISABLE_AUTO_POWER_ON
 		rContext->readerState->cardAtrLength = 0;
@@ -272,7 +269,7 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 			readerState = SCARD_PRESENT | SCARD_SWALLOWED;
 			rContext->powerState = POWER_STATE_UNPOWERED;
 			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
-			Log3(PCSC_LOG_ERROR, "Error powering up card: %d 0x%04X", rv, rv);
+			Log3(PCSC_LOG_ERROR, "Error powering up card: %ld 0x%04lX", rv, rv);
 		}
 #endif
 
@@ -432,7 +429,7 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 				timeout = PCSCLITE_STATUS_EVENT_TIMEOUT;
 
 			ret = rContext->pthCardEvent(rContext->slot, timeout);
-			if (IFD_NO_SUCH_DEVICE == ret)
+			if (IFD_SUCCESS != ret)
 				(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 		}
 		else
