@@ -3,10 +3,10 @@
  *
  * Copyright (C) 2000-2003
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: hotplug_generic.c 5047 2010-06-29 14:39:24Z rousseau $
+ * $Id: hotplug_generic.c 5711 2011-05-05 09:02:08Z rousseau $
  */
 
 /**
@@ -27,7 +27,7 @@
 #define FALSE 0
 #endif
 
-#if !defined(__APPLE__) && !defined(HAVE_LIBUSB) && !defined(__linux__) && !defined(HAVE_LIBHAL)
+#if !defined(__APPLE__) && !defined(HAVE_LIBUSB) && !defined(__linux__) && !defined(HAVE_LIBUDEV)
 
 LONG HPSearchHotPluggables(void)
 {
