@@ -3,14 +3,14 @@
  *
  * Copyright (C) 2002-2004
  *  Stephen M. Webb <stephenw@cryptocard.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  * Copyright (C) 2002
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003
  *  Antti Tapaninen
  *
- * $Id: hotplug_macosx.c 5508 2011-01-14 08:44:52Z rousseau $
+ * $Id: hotplug_macosx.c 5868 2011-07-09 11:59:09Z rousseau $
  */
 
 /**
@@ -181,7 +181,7 @@ static HPDriverVector HPDriversGetFromDirectory(const char *driverBundlePath)
 	 *  and HPDriverVectorRelease:376 */
 	readersNumber++;
 
-	bundleVector = (HPDriver *) calloc(readersNumber, sizeof(HPDriver));
+	bundleVector = calloc(readersNumber, sizeof(HPDriver));
 	if (!bundleVector)
 	{
 		Log1(PCSC_LOG_ERROR, "memory allocation failure");
@@ -243,7 +243,7 @@ static HPDriverVector HPDriversGetFromDirectory(const char *driverBundlePath)
 			if (reader_nb != CFArrayGetCount(productArray))
 			{
 				Log3(PCSC_LOG_ERROR,
-					"Malformed Info.plist: %d vendors and %d products",
+					"Malformed Info.plist: %d vendors and %ld products",
 					reader_nb, CFArrayGetCount(productArray));
 				return bundleVector;
 			}
@@ -251,7 +251,7 @@ static HPDriverVector HPDriversGetFromDirectory(const char *driverBundlePath)
 			if (reader_nb != CFArrayGetCount(friendlyNameArray))
 			{
 				Log3(PCSC_LOG_ERROR,
-					"Malformed Info.plist: %d vendors and %d friendlynames",
+					"Malformed Info.plist: %d vendors and %ld friendlynames",
 					reader_nb, CFArrayGetCount(friendlyNameArray));
 				return bundleVector;
 			}
@@ -349,7 +349,7 @@ static HPDriver *HPDriverCopy(HPDriver * rhs)
 	if (!rhs)
 		return NULL;
 
-	HPDriver *newDriverBundle = (HPDriver *) calloc(1, sizeof(HPDriver));
+	HPDriver *newDriverBundle = calloc(1, sizeof(HPDriver));
 
 	if (!newDriverBundle)
 		return NULL;
@@ -396,7 +396,7 @@ static void HPDriverVectorRelease(HPDriverVector driverBundleVector)
 static HPDeviceList
 HPDeviceListInsert(HPDeviceList list, HPDriver * bundle, UInt32 address)
 {
-	HPDevice *newReader = (HPDevice *) calloc(1, sizeof(HPDevice));
+	HPDevice *newReader = calloc(1, sizeof(HPDevice));
 
 	if (!newReader)
 	{
