@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2004
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rouseau@free.fr>
  *
- * $Id: ifdwrapper.h 4727 2010-02-09 10:56:49Z rousseau $
+ * $Id: ifdwrapper.h 5434 2010-12-08 14:13:21Z rousseau $
  */
 
 /**
@@ -18,16 +18,10 @@
 #ifndef __ifdwrapper_h__
 #define __ifdwrapper_h__
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 	LONG IFDOpenIFD(READER_CONTEXT *);
 	LONG IFDCloseIFD(READER_CONTEXT *);
 	LONG IFDPowerICC(READER_CONTEXT *, DWORD, PUCHAR, /*@out@*/ PDWORD);
-	LONG IFDStatusICC(READER_CONTEXT *, /*@out@*/ PDWORD, /*@out@*/ PUCHAR,
-		/*@out@*/ PDWORD);
+	LONG IFDStatusICC(READER_CONTEXT *, /*@out@*/ PDWORD);
 	LONG IFDControl_v2(READER_CONTEXT *, PUCHAR, DWORD, /*@out@*/ PUCHAR,
 		PDWORD);
 	LONG IFDControl(READER_CONTEXT *, DWORD, LPCVOID, DWORD, LPVOID,
@@ -38,8 +32,4 @@ extern "C"
 	LONG IFDSetCapabilities(READER_CONTEXT *, DWORD, DWORD, PUCHAR);
 	LONG IFDGetCapabilities(READER_CONTEXT *, DWORD, PDWORD, /*@out@*/ PUCHAR);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif							/* __ifdwrapper_h__ */
