@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 5343 2010-10-22 11:52:22Z rousseau $
+ * $Id: pcscdaemon.c 6105 2011-11-14 10:19:44Z rousseau $
  */
 
 /**
@@ -37,6 +37,7 @@
 #include "pcsclite.h"
 #include "pcscd.h"
 #include "debuglog.h"
+#include "sd-daemon.h"
 #include "winscard_msg.h"
 #include "winscard_svc.h"
 #include "sys_generic.h"
@@ -54,6 +55,7 @@
 char AraKiri = FALSE;
 static char Init = TRUE;
 char AutoExit = FALSE;
+char SocketActivated = FALSE;
 static int ExitValue = EXIT_FAILURE;
 int HPForceReaderPolling = 0;
 static int pipefd[] = {-1, -1};
@@ -82,8 +84,6 @@ static void SVCServiceRunLoop(void)
 	LONG rv;
 	uint32_t dwClientID;	/* Connection ID used to reference the Client */
 
-	rv = 0;
-
 	while (TRUE)
 	{
 		switch (rsp = ProcessEventsServer(&dwClientID))
@@ -93,7 +93,7 @@ static void SVCServiceRunLoop(void)
 			Log2(PCSC_LOG_DEBUG, "A new context thread creation is requested: %d", dwClientID);
 			rv = CreateContextThread(&dwClientID);
 
- 			if (rv != SCARD_S_SUCCESS)
+			if (rv != SCARD_S_SUCCESS)
 				Log1(PCSC_LOG_ERROR, "Problem during the context thread creation");
 			break;
 
@@ -154,6 +154,7 @@ int main(int argc, char **argv)
 	static struct option long_options[] = {
 		{"config", 1, NULL, 'c'},
 		{"foreground", 0, NULL, 'f'},
+		{"color", 0, NULL, 'T'},
 		{"help", 0, NULL, 'h'},
 		{"version", 0, NULL, 'v'},
 		{"apdu", 0, NULL, 'a'},
@@ -170,9 +171,8 @@ int main(int argc, char **argv)
 		{NULL, 0, NULL, 0}
 	};
 #endif
-#define OPT_STRING "c:fdhvaeCHt:r:s:x"
+#define OPT_STRING "c:fTdhvaeCHt:r:s:x"
 
-	rv = 0;
 	newReaderConfig = NULL;
 	setToForeground = FALSE;
 	HotPlug = FALSE;
@@ -227,10 +227,15 @@ int main(int argc, char **argv)
 
 			case 'f':
 				setToForeground = TRUE;
-				/* debug to stderr instead of default syslog */
-				DebugLogSetLogType(DEBUGLOG_STDERR_DEBUG);
+				/* debug to stdout instead of default syslog */
+				DebugLogSetLogType(DEBUGLOG_STDOUT_DEBUG);
 				Log1(PCSC_LOG_INFO,
-					"pcscd set to foreground with debug send to stderr");
+					"pcscd set to foreground with debug send to stdout");
+				break;
+
+			case 'T':
+				DebugLogSetLogType(DEBUGLOG_STDOUT_COLOR_DEBUG);
+				Log1(PCSC_LOG_INFO, "Force colored logs");
 				break;
 
 			case 'd':
@@ -263,8 +268,8 @@ int main(int argc, char **argv)
 				break;
 
 			case 'H':
-				/* debug to stderr instead of default syslog */
-				DebugLogSetLogType(DEBUGLOG_STDERR_DEBUG);
+				/* debug to stdout instead of default syslog */
+				DebugLogSetLogType(DEBUGLOG_STDOUT_DEBUG);
 				HotPlug = TRUE;
 				break;
 
@@ -312,6 +317,26 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
+	/*
+	 * Check if systemd passed us any file descriptors
+	 */
+	rv = sd_listen_fds(0);
+	if (rv > 1)
+	{
+		Log1(PCSC_LOG_CRITICAL, "Too many file descriptors received");
+		return EXIT_FAILURE;
+	}
+	else
+	{
+		if (rv == 1)
+		{
+			SocketActivated = TRUE;
+			Log1(PCSC_LOG_INFO, "Started by systemd");
+		}
+		else
+			SocketActivated = FALSE;
+	}
+
 	/*
 	 * test the presence of /var/run/pcscd/pcscd.comm
 	 */
@@ -362,17 +387,6 @@ int main(int argc, char **argv)
 				Log1(PCSC_LOG_CRITICAL, "Hotplug failed");
 				return EXIT_FAILURE;
 			}
-
-			Log1(PCSC_LOG_CRITICAL,
-				"file " PCSCLITE_CSOCK_NAME " already exists.");
-			Log1(PCSC_LOG_CRITICAL,
-				"Maybe another pcscd is running?");
-			Log1(PCSC_LOG_CRITICAL,
-				"I can't read process pid from " PCSCLITE_RUN_PID);
-			Log1(PCSC_LOG_CRITICAL, "Remove " PCSCLITE_CSOCK_NAME);
-			Log1(PCSC_LOG_CRITICAL,
-				"if pcscd is not running to clear this message.");
-			return EXIT_FAILURE;
 		}
 	}
 	else
@@ -386,24 +400,6 @@ int main(int argc, char **argv)
 	 * root ("/") */
 	(void)chdir("/");
 
-	if (AutoExit)
-	{
-		int pid;
-
-		/* create a new session so that Ctrl-C on the application will
-		 * not also quit pcscd */
-		setsid();
-
-		/* fork() so that pcscd always return in --auto-exit mode */
-		pid = fork();
-		if (-1 == pid )
-			Log2(PCSC_LOG_CRITICAL, "fork() failed: %s", strerror(errno));
-
-		if (pid)
-			/* father */
-			return EXIT_SUCCESS;
-	}
-
 	/*
 	 * If this is set to one the user has asked it not to fork
 	 */
@@ -462,8 +458,8 @@ int main(int argc, char **argv)
 	 * signal_trap() does just set a global variable used by the main loop
 	 */
 	(void)signal(SIGQUIT, signal_trap);
-	(void)signal(SIGTERM, signal_trap);
-	(void)signal(SIGINT, signal_trap);
+	(void)signal(SIGTERM, signal_trap); /* default kill signal & init round 1 */
+	(void)signal(SIGINT, signal_trap);	/* sent by Ctrl-C */
 
 	/* exits on SIGALARM to allow pcscd to suicide if not used */
 	(void)signal(SIGALRM, signal_trap);
@@ -490,37 +486,6 @@ int main(int argc, char **argv)
 		(void)chmod(PCSCLITE_IPC_DIR, mode);
 	}
 
-	/*
-	 * Record our pid to make it easier
-	 * to kill the correct pcscd
-	 */
-	{
-		int f;
-		int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
-
-		f = open(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, mode);
-		if (f != -1)
-		{
-			char pid[PID_ASCII_SIZE];
-
-			(void)snprintf(pid, sizeof(pid), "%u\n", (unsigned) getpid());
-			(void)write(f, pid, strlen(pid));
-			(void)close(f);
-
-			/* set mode so that the file is world readable even is umask is
-			 * restrictive
-			 * The file is used by libpcsclite */
-			(void)chmod(PCSCLITE_RUN_PID, mode);
-		}
-		else
-			Log2(PCSC_LOG_CRITICAL, "cannot create " PCSCLITE_RUN_PID ": %s",
-				strerror(errno));
-	}
-
-	/* cleanly remove /var/run/pcscd/pcsc.* files when exiting */
-	if (atexit(at_exit))
-		Log2(PCSC_LOG_CRITICAL, "atexit() failed: %s", strerror(errno));
-
 	/*
 	 * Allocate memory for reader structures
 	 */
@@ -552,6 +517,35 @@ int main(int argc, char **argv)
 
 	Log1(PCSC_LOG_INFO, "pcsc-lite " VERSION " daemon ready.");
 
+	/*
+	 * Record our pid to make it easier
+	 * to kill the correct pcscd
+	 *
+	 * Do not fork after this point or the stored pid will be wrong
+	 */
+	{
+		int f;
+		int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+
+		f = open(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, mode);
+		if (f != -1)
+		{
+			char pid[PID_ASCII_SIZE];
+
+			(void)snprintf(pid, sizeof(pid), "%u\n", (unsigned) getpid());
+			(void)write(f, pid, strlen(pid));
+			(void)close(f);
+
+			/* set mode so that the file is world readable even is umask is
+			 * restrictive
+			 * The file is used by libpcsclite */
+			(void)chmod(PCSCLITE_RUN_PID, mode);
+		}
+		else
+			Log2(PCSC_LOG_CRITICAL, "cannot create " PCSCLITE_RUN_PID ": %s",
+				strerror(errno));
+	}
+
 	/*
 	 * post initialistion
 	 */
@@ -565,7 +559,11 @@ int main(int argc, char **argv)
 	/*
 	 * Initialize the comm structure
 	 */
-	rv = InitializeSocket();
+	if (SocketActivated)
+		rv = ListenExistingSocket(SD_LISTEN_FDS_START + 0);
+	else
+		rv = InitializeSocket();
+
 	if (rv)
 	{
 		Log1(PCSC_LOG_CRITICAL, "Error initializing pcscd.");
@@ -642,17 +640,20 @@ static void at_exit(void)
 		close(pipefd[1]);
 	}
 
-	_exit(ExitValue);
+	exit(ExitValue);
 }
 
 static void clean_temp_files(void)
 {
 	int rv;
 
-	rv = remove(PCSCLITE_CSOCK_NAME);
-	if (rv != 0)
-		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_CSOCK_NAME ": %s",
-			strerror(errno));
+	if (!SocketActivated)
+	{
+		rv = remove(PCSCLITE_CSOCK_NAME);
+		if (rv != 0)
+			Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_CSOCK_NAME ": %s",
+				strerror(errno));
+	}
 
 	rv = remove(PCSCLITE_RUN_PID);
 	if (rv != 0)
@@ -678,6 +679,20 @@ static void signal_trap(int sig)
 {
 	Log2(PCSC_LOG_INFO, "Received signal: %d", sig);
 
+	/* do not wait if asked to terminate
+	 * avoids waiting after the reader(s) in shutdown for example */
+	if (SIGTERM == sig)
+	{
+		Log1(PCSC_LOG_INFO, "Direct suicide");
+		at_exit();
+	}
+
+	if (SIGALRM == sig)
+	{
+		/* normal exit without error */
+		ExitValue = EXIT_SUCCESS;
+	}
+
 	/* the signal handler is called several times for the same Ctrl-C */
 	if (AraKiri == FALSE)
 	{
@@ -712,7 +727,7 @@ static void print_version (void)
 {
 	printf("%s version %s.\n",  PACKAGE, VERSION);
 	printf("Copyright (C) 1999-2002 by David Corcoran <corcoran@linuxnet.com>.\n");
-	printf("Copyright (C) 2001-2010 by Ludovic Rousseau <ludovic.rousseau@free.fr>.\n");
+	printf("Copyright (C) 2001-2011 by Ludovic Rousseau <ludovic.rousseau@free.fr>.\n");
 	printf("Copyright (C) 2003-2004 by Damien Sauveron <sauveron@labri.fr>.\n");
 	printf("Report bugs to <muscle@lists.musclecard.com>.\n");
 
@@ -727,29 +742,35 @@ static void print_usage (char const * const progname)
 	printf("  -a, --apdu		log APDU commands and results\n");
 	printf("  -c, --config		path to reader.conf\n");
 	printf("  -f, --foreground	run in foreground (no daemon),\n");
-	printf("			send logs to stderr instead of syslog\n");
+	printf("			send logs to stdout instead of syslog\n");
+	printf("  -T, --color		force use of colored logs\n");
 	printf("  -h, --help		display usage information\n");
 	printf("  -H, --hotplug		ask the daemon to rescan the available readers\n");
 	printf("  -v, --version		display the program version number\n");
-	printf("  -d, --debug	 	display lower level debug messages\n");
-	printf("      --info	 	display info level debug messages (default level)\n");
-	printf("  -e  --error	 	display error level debug messages\n");
-	printf("  -C  --critical 	display critical only level debug messages\n");
+	printf("  -d, --debug		display lower level debug messages\n");
+	printf("      --info		display info level debug messages\n");
+	printf("  -e  --error		display error level debug messages (default level)\n");
+	printf("  -C  --critical	display critical only level debug messages\n");
 	printf("  --force-reader-polling ignore the IFD_GENERATE_HOTPLUG reader capability\n");
 	printf("  -t, --max-thread	maximum number of threads (default %d)\n", PCSC_MAX_CONTEXT_THREADS);
 	printf("  -s, --max-card-handle-per-thread	maximum number of card handle per thread (default: %d)\n", PCSC_MAX_CONTEXT_CARD_HANDLES);
 	printf("  -r, --max-card-handle-per-reader	maximum number of card handle per reader (default: %d)\n", PCSC_MAX_READER_HANDLES);
+	printf("  -x, --auto-exit	pcscd will quit after %d seconds of inactivity\n", TIME_BEFORE_SUICIDE);
 #else
 	printf("  -a    log APDU commands and results\n");
-	printf("  -c 	path to reader.conf\n");
-	printf("  -f	run in foreground (no daemon), send logs to stderr instead of syslog\n");
-	printf("  -d 	display debug messages. Output may be:\n");
-	printf("  -h 	display usage information\n");
+	printf("  -c	path to reader.conf\n");
+	printf("  -f	run in foreground (no daemon), send logs to stdout instead of syslog\n");
+	printf("  -T    force use of colored logs\n");
+	printf("  -d	display debug messages.\n");
+	printf("  -e	display error messages (default level).\n");
+	printf("  -C	display critical messages.\n");
+	printf("  -h	display usage information\n");
 	printf("  -H	ask the daemon to rescan the available readers\n");
-	printf("  -v 	display the program version number\n");
+	printf("  -v	display the program version number\n");
 	printf("  -t    maximum number of threads\n");
 	printf("  -s    maximum number of card handle per thread\n");
 	printf("  -r    maximum number of card handle per reader\n");
+	printf("  -x	pcscd will quit after %d seconds of inactivity\n", TIME_BEFORE_SUICIDE);
 #endif
 }
 
