@@ -10,7 +10,7 @@
  * Copyright (C) 2009
  *  Jean-Luc Giraud <jlgiraud@googlemail.com>
  *
- * $Id: readerfactory.c 6129 2011-12-05 13:57:11Z rousseau $
+ * $Id: readerfactory.c 6356 2012-06-25 13:38:46Z rousseau $
  */
 
 /**
@@ -106,7 +106,7 @@ LONG RFAllocateReaderSpace(unsigned int customMaxReaderHandles)
 	return EHInitializeEventStructures();
 }
 
-LONG RFAddReader(const char *readerName, int port, const char *library,
+LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 	const char *device)
 {
 	DWORD dwContext = 0, dwGetSize;
@@ -114,17 +114,22 @@ LONG RFAddReader(const char *readerName, int port, const char *library,
 	LONG rv, parentNode;
 	int i, j;
 	int lrv = 0;
+	char *readerName = NULL;
 
-	if ((readerName == NULL) || (library == NULL) || (device == NULL))
+	if ((readerNameLong == NULL) || (library == NULL) || (device == NULL))
 		return SCARD_E_INVALID_VALUE;
 
+	/* allocate memory that is automatically freed */
+	readerName = alloca(strlen(readerNameLong));
+	strcpy(readerName, readerNameLong);
+
 	/* Reader name too long? also count " 00 00"*/
 	if (strlen(readerName) > MAX_READERNAME - sizeof(" 00 00"))
 	{
 		Log3(PCSC_LOG_ERROR,
-			"Reader name too long: %zd chars instead of max %zd",
+			"Reader name too long: %zd chars instead of max %zd. Truncating!",
 			strlen(readerName), MAX_READERNAME - sizeof(" 00 00"));
-		return SCARD_E_INVALID_VALUE;
+		readerName[MAX_READERNAME - sizeof(" 00 00")] = '\0';
 	}
 
 	/* Same name, same port - duplicate reader cannot be used */
@@ -189,8 +194,6 @@ LONG RFAddReader(const char *readerName, int port, const char *library,
 	sReadersContexts[dwContext]->pFeeds = NULL;
 	sReadersContexts[dwContext]->pMutex = NULL;
 	sReadersContexts[dwContext]->pthCardEvent = NULL;
-	sReadersContexts[dwContext]->dwIdentity =
-		(dwContext + 1) << IDENTITY_SHIFT;
 
 	lrv = list_init(&sReadersContexts[dwContext]->handlesList);
 	if (lrv < 0)
@@ -386,8 +389,6 @@ LONG RFAddReader(const char *readerName, int port, const char *library,
 		sReadersContexts[dwContextB]->contexts = 0;
 		sReadersContexts[dwContextB]->hLockId = 0;
 		sReadersContexts[dwContextB]->LockCount = 0;
-		sReadersContexts[dwContextB]->dwIdentity =
-			(dwContextB + 1) << IDENTITY_SHIFT;
 
 		lrv = list_init(&sReadersContexts[dwContextB]->handlesList);
 		if (lrv < 0)
@@ -522,7 +523,6 @@ LONG RFRemoveReader(const char *readerName, int port)
 		sContext->hLockId = 0;
 		sContext->LockCount = 0;
 		sContext->vHandle = NULL;
-		sContext->dwIdentity = 0;
 
 		(void)pthread_mutex_lock(&sContext->handlesList_lock);
 		while (list_size(&(sContext->handlesList)) != 0)
@@ -700,8 +700,9 @@ LONG RFReaderInfoNamePort(int port, const char *readerName,
 			tmplen = strlen(lpcStripReader);
 			lpcStripReader[tmplen - 6] = 0;
 
-			if ((strcmp(readerName, lpcStripReader) == 0) &&
-				(port == sReadersContexts[i]->port))
+			/* Compare only the significant part of the reader name */
+			if ((strncmp(readerName, lpcStripReader, MAX_READERNAME - sizeof(" 00 00")) == 0)
+				&& (port == sReadersContexts[i]->port))
 			{
 				*sReader = sReadersContexts[i];
 				return SCARD_S_SUCCESS;
@@ -712,20 +713,24 @@ LONG RFReaderInfoNamePort(int port, const char *readerName,
 	return SCARD_E_INVALID_VALUE;
 }
 
-LONG RFReaderInfoById(DWORD dwIdentity, READER_CONTEXT * * sReader)
+LONG RFReaderInfoById(SCARDHANDLE hCard, READER_CONTEXT * * sReader)
 {
 	int i;
 
-	/* Strip off the lower nibble and get the identity */
-	dwIdentity = dwIdentity >> IDENTITY_SHIFT;
-	dwIdentity = dwIdentity << IDENTITY_SHIFT;
-
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if (dwIdentity == sReadersContexts[i]->dwIdentity)
+		if (sReadersContexts[i]->vHandle != 0)
 		{
-			*sReader = sReadersContexts[i];
-			return SCARD_S_SUCCESS;
+			RDR_CLIHANDLES * currentHandle;
+			(void)pthread_mutex_lock(&sReadersContexts[i]->handlesList_lock);
+			currentHandle = list_seek(&sReadersContexts[i]->handlesList,
+				&hCard);
+			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
+			if (currentHandle != NULL)
+			{
+				*sReader = sReadersContexts[i];
+				return SCARD_S_SUCCESS;
+			}
 		}
 	}
 
@@ -1011,68 +1016,27 @@ LONG RFUnInitializeReader(READER_CONTEXT * rContext)
 
 SCARDHANDLE RFCreateReaderHandle(READER_CONTEXT * rContext)
 {
-	USHORT randHandle;
+	SCARDHANDLE randHandle;
+	READER_CONTEXT *dummy_reader;
+	LONG ret;
 
-	/* Create a random handle with 16 bits check to see if it already is
-	 * used. */
-	/* FIXME: THIS IS NOT STRONG ENOUGH: A 128-bit token should be
-	 * generated.  The client and server would associate token and hCard
-	 * for authentication. */
-	randHandle = SYS_RandomInt(10, 65000);
-
-	int i;
-again:
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
+	do
 	{
-		if (sReadersContexts[i]->vHandle != 0)
-		{
-			RDR_CLIHANDLES *currentHandle;
-			list_t * l = &sReadersContexts[i]->handlesList;
+		/* Create a random handle with 32 bits check to see if it already is
+		 * used. */
+		/* FIXME: THIS IS NOT STRONG ENOUGH: A 128-bit token should be
+		 * generated.  The client and server would associate token and hCard
+		 * for authentication. */
+		randHandle = SYS_RandomInt(0, -1);
 
-			(void)pthread_mutex_lock(&sReadersContexts[i]->handlesList_lock);
-			list_iterator_start(l);
-			while (list_iterator_hasnext(l))
-			{
-				currentHandle = list_iterator_next(l);
-				if (((LONG)rContext->dwIdentity + randHandle) ==
-					(currentHandle->hCard))
-				{
-					/* Get a new handle and loop again */
-					randHandle = SYS_RandomInt(10, 65000);
-					list_iterator_stop(l);
-					(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
-					goto again;
-				}
-			}
-			list_iterator_stop(l);
-			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
-		}
+		/* do we already use this hCard somewhere? */
+		ret = RFReaderInfoById(randHandle, &dummy_reader);
 	}
+	while (SCARD_S_SUCCESS == ret);
 
 	/* Once the for loop is completed w/o restart a good handle was
 	 * found and the loop can be exited. */
-	return rContext->dwIdentity + randHandle;
-}
-
-LONG RFFindReaderHandle(SCARDHANDLE hCard)
-{
-	int i;
-
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		if (sReadersContexts[i]->vHandle != 0)
-		{
-			RDR_CLIHANDLES * currentHandle;
-			(void)pthread_mutex_lock(&sReadersContexts[i]->handlesList_lock);
-			currentHandle = list_seek(&sReadersContexts[i]->handlesList,
-				&hCard);
-			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
-			if (currentHandle != NULL)
-				return SCARD_S_SUCCESS;
-		}
-	}
-
-	return SCARD_E_INVALID_HANDLE;
+	return randHandle;
 }
 
 LONG RFDestroyReaderHandle(/*@unused@*/ SCARDHANDLE hCard)
