@@ -6,7 +6,7 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.h 5900 2011-08-21 13:53:33Z rousseau $
+ * $Id: readerfactory.h 6321 2012-06-05 09:07:46Z rousseau $
  */
 
 /**
@@ -103,7 +103,6 @@
 		int port;				/**< Port ID */
 		int slot;				/**< Current Reader Slot */
 		SCARDHANDLE hLockId;	/**< Lock Id */
-		DWORD dwIdentity;		/**< Shared ID High Nibble */
 		int LockCount;			/**< number of recursive locks */
 		int32_t contexts;		/**< Number of open contexts */
 		int * pFeeds;			/**< Number of shared client to lib */
@@ -124,7 +123,7 @@
 	LONG RFSetReaderName(READER_CONTEXT *, const char *, const char *, int);
 	LONG RFReaderInfo(const char *, /*@out@*/ struct ReaderContext **);
 	LONG RFReaderInfoNamePort(int, const char *, /*@out@*/ struct ReaderContext **);
-	LONG RFReaderInfoById(DWORD, /*@out@*/ struct ReaderContext **);
+	LONG RFReaderInfoById(SCARDHANDLE, /*@out@*/ struct ReaderContext **);
 	LONG RFCheckSharing(SCARDHANDLE, READER_CONTEXT *);
 	LONG RFLockSharing(SCARDHANDLE, READER_CONTEXT *);
 	LONG RFUnlockSharing(SCARDHANDLE, READER_CONTEXT *);
@@ -138,7 +137,6 @@
 	SCARDHANDLE RFCreateReaderHandle(READER_CONTEXT *);
 	LONG RFDestroyReaderHandle(SCARDHANDLE hCard);
 	LONG RFAddReaderHandle(READER_CONTEXT *, SCARDHANDLE);
-	LONG RFFindReaderHandle(SCARDHANDLE);
 	LONG RFRemoveReaderHandle(READER_CONTEXT *, SCARDHANDLE);
 	LONG RFSetReaderEventState(READER_CONTEXT *, DWORD);
 	LONG RFCheckReaderEventState(READER_CONTEXT *, SCARDHANDLE);
