@@ -16,7 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* $Id: libpcscspy.c 6177 2012-01-15 17:20:00Z rousseau $ */
+/* $Id: libpcscspy.c 6234 2012-02-28 14:38:13Z rousseau $ */
 
 #include <dlfcn.h>
 #include <stdio.h>
@@ -34,6 +34,14 @@
 
 #define DEBUG
 
+#ifdef __APPLE__
+#define SCardControl SCardControl132
+
+PCSC_API int32_t SCardControl132(SCARDHANDLE hCard, uint32_t dwControlCode,
+        const void *pbSendBuffer, uint32_t cbSendLength,
+        void *pbRecvBuffer, uint32_t cbRecvLength, uint32_t *lpBytesReturned);
+#endif
+
 /* function prototypes */
 
 #define p_SCardEstablishContext(fct) LONG(fct)(DWORD dwScope, LPCVOID pvReserved1, LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
@@ -80,6 +88,11 @@ static LONG internal_error(void)
 	return SCARD_F_INTERNAL_ERROR;
 }
 
+static const char * internal_stringify_error(void)
+{
+	return "No spy pcsc_stringify_error() function";
+}
+
 /* contains pointers to real functions */
 static struct
 {
@@ -122,7 +135,7 @@ static struct
 	.SCardCancel = (p_SCardCancel(*))internal_error,
 	.SCardGetAttrib = (p_SCardGetAttrib(*))internal_error,
 	.SCardSetAttrib = (p_SCardSetAttrib(*))internal_error,
-	.pcsc_stringify_error = (p_pcsc_stringify_error(*))internal_error
+	.pcsc_stringify_error = (p_pcsc_stringify_error(*))internal_stringify_error
 };
 
 #define LOG log_line("%s:%d", __FILE__, __LINE__)
@@ -215,7 +228,7 @@ static void spy_long(long arg)
 	spy_line("0x%08lX", arg);
 }
 
-static void spy_ptr_long(long *arg)
+static void spy_ptr_long(LONG *arg)
 {
 	if (arg)
 		spy_line("0x%08lX", *arg);
@@ -223,7 +236,7 @@ static void spy_ptr_long(long *arg)
 		spy_line("NULL");
 }
 
-static void spy_ptr_ulong(unsigned long *arg)
+static void spy_ptr_ulong(ULONG *arg)
 {
 	if (arg)
 		spy_line("0x%08lX", *arg);
@@ -266,7 +279,7 @@ static void spy_str(const char *str)
     spy_line("%s", str);
 }
 
-static void spy_n_str(const char *str, unsigned long *len, int autoallocate)
+static void spy_n_str(const char *str, ULONG *len, int autoallocate)
 {
     spy_ptr_ulong(len);
     if (NULL == len)
@@ -289,7 +302,7 @@ static void spy_n_str(const char *str, unsigned long *len, int autoallocate)
 
             do
             {
-                spy_line("%s", s);
+				spy_line("%s", s);
                 length += strlen(s)+1;
                 s += strlen(s)+1;
             } while(length < *len);
@@ -317,8 +330,19 @@ static void spy_readerstate(SCARD_READERSTATE * rgReaderStates, int cReaders)
 static LONG load_lib(void)
 {
 
+#ifdef __APPLE__
+/* We should be able to directly use this
+ * #define LIBPCSC_NOSPY "/System/Library/Frameworks/PCSC.framework/PCSC"
+ * but for a yet unknown reason the dlsym() returns symbols from the spy
+ * library and not from the framework.
+ * Just copying the framework in /tmp does solve the problem.
+ */
+#define LIBPCSC_NOSPY "/tmp/PCSC"
+#define LIBPCSC "/tmp/PCSC"
+#else
 #define LIBPCSC_NOSPY "libpcsclite_nospy.so.1"
 #define LIBPCSC "libpcsclite.so.1"
+#endif
 
 	/* first try to load the NOSPY library
 	 * this is used for programs doing an explicit dlopen like
@@ -339,6 +363,12 @@ static LONG load_lib(void)
 
 #define get_symbol(s) do { spy.s = dlsym(Lib_handle, #s); if (NULL == spy.s) { log_line("%s", dlerror()); return SCARD_F_INTERNAL_ERROR; } } while (0)
 
+	if (SCardEstablishContext == dlsym(Lib_handle, "SCardEstablishContext"))
+	{
+		log_line("Symbols dlsym error");
+		return SCARD_F_INTERNAL_ERROR;
+	}
+
 	get_symbol(SCardEstablishContext);
 	get_symbol(SCardReleaseContext);
 	get_symbol(SCardIsValidContext);
@@ -353,7 +383,9 @@ static LONG load_lib(void)
 	get_symbol(SCardTransmit);
 	get_symbol(SCardListReaderGroups);
 	get_symbol(SCardListReaders);
-	get_symbol(SCardFreeMemory);
+	/* Mac OS X do not have SCardFreeMemory() */
+	if (dlsym(Lib_handle, "SCardFreeMemory"))
+		get_symbol(SCardFreeMemory);
 	get_symbol(SCardCancel);
 	get_symbol(SCardGetAttrib);
 	get_symbol(SCardSetAttrib);
