@@ -6,7 +6,7 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 5962 2011-09-24 08:24:34Z rousseau $
+ * $Id: winscard.c 6322 2012-06-05 09:17:50Z rousseau $
  */
 
 /**
@@ -188,7 +188,7 @@ LONG SCardEstablishContext(DWORD dwScope, /*@unused@*/ LPCVOID pvReserved1,
 	 * identified by clients and distinguished from others
 	 */
 
-	*phContext = (PCSCLITE_SVC_IDENTITY + SYS_RandomInt(1, 65535));
+	*phContext = SYS_RandomInt(0, -1);
 
 	Log2(PCSC_LOG_DEBUG, "Establishing Context: 0x%lX", *phContext);
 
@@ -528,10 +528,6 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
@@ -774,10 +770,6 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	if ((dwDisposition != SCARD_LEAVE_CARD)
 		&& (dwDisposition != SCARD_UNPOWER_CARD)
 		&& (dwDisposition != SCARD_RESET_CARD)
@@ -1030,10 +1022,6 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure some event has not occurred
 	 */
@@ -1075,10 +1063,6 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure some event has not occurred
 	 */
@@ -1283,10 +1267,6 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	if (IFD_HVERSION_2_0 == rContext->version)
 	{
 		/* we must wrap a API 3.0 client in an API 2.0 driver */
@@ -1330,10 +1310,6 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure some event has not occurred
 	 */
@@ -1407,10 +1383,6 @@ LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure some event has not occurred
 	 */
@@ -1469,10 +1441,6 @@ LONG SCardTransmit(SCARDHANDLE hCard, const SCARD_IO_REQUEST *pioSendPci,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	/*
 	 * Make sure some event has not occurred
 	 */
