@@ -8,7 +8,7 @@
  * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.c 4952 2010-05-18 13:42:25Z rousseau $
+ * $Id: winscard_msg.c 5867 2011-07-09 11:50:16Z rousseau $
  */
 
 /**
@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <time.h>
 #include <string.h>
+#include <stdlib.h>
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
 #endif
@@ -44,6 +45,34 @@
 #include "winscard_msg.h"
 #include "sys_generic.h"
 #include "utils.h"
+#include "strlcpycat.h"
+
+#ifdef PCSCD
+
+/* functions used by pcscd only */
+
+#else
+
+/* functions used by libpcsclite only */
+
+char *getSocketName(void)
+{
+	static char socketName[sizeof(struct sockaddr_un)];
+
+	if ('\0' == socketName[0])
+	{
+		/* socket name not yet initialized */
+		char *socketNameEnv;
+
+		socketNameEnv = getenv("PCSCLITE_CSOCK_NAME");
+		if (socketNameEnv)
+			strlcpy(socketName, socketNameEnv, sizeof(socketName));
+		else
+			strlcpy(socketName, PCSCLITE_CSOCK_NAME, sizeof(socketName));
+	}
+
+	return socketName;
+}
 
 /**
  * @brief Prepares a communication channel for the client to talk to the server.
@@ -63,6 +92,7 @@ INTERNAL int ClientSetupSession(uint32_t *pdwClientID)
 	struct sockaddr_un svc_addr;
 	int one;
 	int ret;
+	char *socketName;
 
 	ret = socket(PF_UNIX, SOCK_STREAM, 0);
 	if (ret < 0)
@@ -73,15 +103,15 @@ INTERNAL int ClientSetupSession(uint32_t *pdwClientID)
 	}
 	*pdwClientID = ret;
 
+	socketName = getSocketName();
 	svc_addr.sun_family = AF_UNIX;
-	strncpy(svc_addr.sun_path, PCSCLITE_CSOCK_NAME,
-		sizeof(svc_addr.sun_path));
+	strncpy(svc_addr.sun_path, socketName, sizeof(svc_addr.sun_path));
 
 	if (connect(*pdwClientID, (struct sockaddr *) &svc_addr,
 			sizeof(svc_addr.sun_family) + strlen(svc_addr.sun_path) + 1) < 0)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: connect to client socket %s: %s",
-			PCSCLITE_CSOCK_NAME, strerror(errno));
+			socketName, strerror(errno));
 		(void)close(*pdwClientID);
 		return -1;
 	}
@@ -90,7 +120,7 @@ INTERNAL int ClientSetupSession(uint32_t *pdwClientID)
 	if (ioctl(*pdwClientID, FIONBIO, &one) < 0)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: cannot set socket %s nonblocking: %s",
-			PCSCLITE_CSOCK_NAME, strerror(errno));
+			socketName, strerror(errno));
 		(void)close(*pdwClientID);
 		return -1;
 	}
@@ -110,191 +140,6 @@ INTERNAL int ClientCloseSession(uint32_t dwClientID)
 	return close(dwClientID);
 }
 
-/**
- * @brief Sends a menssage from client to server or vice-versa.
- *
- * Writes the message in the shared file \c filedes.
- *
- * @param[in] buffer_void Message to be sent.
- * @param[in] buffer_size Size of the message to send
- * @param[in] filedes Socket handle.
- * @param[in] timeOut Timeout in milliseconds.
- *
- * @retval 0 Success
- * @retval -1 Timeout.
- * @retval -1 Socket is closed.
- * @retval -1 A signal was received.
- */
-INTERNAL int32_t MessageSend(void *buffer_void, uint64_t buffer_size,
-	int32_t filedes)
-{
-	char *buffer = buffer_void;
-
-	/* default is success */
-	int retval = 0;
-
-	/* how many bytes remains to be written */
-	size_t remaining = buffer_size;
-
-	/* repeat until all data is written */
-	while (remaining > 0)
-	{
-		fd_set write_fd;
-		int selret;
-
-		FD_ZERO(&write_fd);
-		FD_SET(filedes, &write_fd);
-
-		selret = select(filedes + 1, NULL, &write_fd, NULL, NULL);
-
-		/* try to write only when the file descriptor is writable */
-		if (selret > 0)
-		{
-			int written;
-
-			if (!FD_ISSET(filedes, &write_fd))
-			{
-				/* very strange situation. it should be an assert really */
-				retval = -1;
-				break;
-			}
-			/* since we are a user library we can't play with signals
-			 * The signals may already be used by the application */
-#ifdef MSG_NOSIGNAL
-			/* Get EPIPE return code instead of SIGPIPE signal
-			 * Works on Linux */
-			written = send(filedes, buffer, remaining, MSG_NOSIGNAL);
-#else
-			/* we may get a SIGPIPE signal if the other side has closed */
-			written = write(filedes, buffer, remaining);
-#endif
-
-			if (written > 0)
-			{
-				/* we wrote something */
-				buffer += written;
-				remaining -= written;
-			} else if (written == 0)
-			{
-				/* peer closed the socket */
-				retval = -1;
-				break;
-			} else
-			{
-				/* we ignore the signals and socket full situations, all
-				 * other errors are fatal */ 
-				if (errno != EINTR && errno != EAGAIN)
-				{
-					retval = -1;
-					break;
-				}
-			}
-		} else if (selret == 0)
-		{
-			/* timeout */
-			retval = -1;
-			break;
-		} else
-		{
-			/* ignore signals */
-			if (errno != EINTR)
-			{
-				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
-					strerror(errno));
-				retval = -1;
-				break;
-			}
-		}
-	}
-
-	return retval;
-}
-
-/**
- * @brief Called by the Client to get the reponse from the server or vice-versa.
- *
- * Reads the message from the file \c filedes.
- *
- * @param[out] buffer_void Message read.
- * @param[in] buffer_size Size to read
- * @param[in] filedes Socket handle.
- *
- * @retval 0 Success.
- * @retval -1 Socket is closed.
- * @retval -1 A signal was received.
- */
-INTERNAL int32_t MessageReceive(void *buffer_void, uint64_t buffer_size,
-	int32_t filedes)
-{
-	char *buffer = buffer_void;
-
-	/* default is success */
-	int retval = 0;
-
-	/* how many bytes we must read */
-	size_t remaining = buffer_size;
-
-	/* repeat until we get the whole message */
-	while (remaining > 0)
-	{
-		fd_set read_fd;
-		int selret;
-
-		FD_ZERO(&read_fd);
-		FD_SET(filedes, &read_fd);
-
-		selret = select(filedes + 1, &read_fd, NULL, NULL, NULL);
-
-		/* try to read only when socket is readable */
-		if (selret > 0)
-		{
-			int readed;
-
-			if (!FD_ISSET(filedes, &read_fd))
-			{
-				/* very strange situation. it should be an assert really */
-				retval = -1;
-				break;
-			}
-			readed = read(filedes, buffer, remaining);
-
-			if (readed > 0)
-			{
-				/* we got something */
-				buffer += readed;
-				remaining -= readed;
-			} else if (readed == 0)
-			{
-				/* peer closed the socket */
-				retval = -1;
-				break;
-			} else
-			{
-				/* we ignore the signals and empty socket situations, all
-				 * other errors are fatal */
-				if (errno != EINTR && errno != EAGAIN)
-				{
-					retval = -1;
-					break;
-				}
-			}
-		}
-		else
-		{
-			/* we ignore signals, all other errors are fatal */
-			if (errno != EINTR)
-			{
-				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
-					strerror(errno));
-				retval = -1;
-				break;
-			}
-		}
-	}
-
-	return retval;
-}
-
 /**
  * @brief Called by the Client to get the reponse from the server or vice-versa.
  *
@@ -306,18 +151,18 @@ INTERNAL int32_t MessageReceive(void *buffer_void, uint64_t buffer_size,
  * @param[in] filedes Socket handle.
  * @param[in] timeOut Timeout in milliseconds.
  *
- * @retval 0 Success.
- * @retval -1 Timeout.
- * @retval -1 Socket is closed.
- * @retval -1 A signal was received.
+ * @retval SCARD_S_SUCCESS Success.
+ * @retval SCARD_E_TIMEOUT Timeout.
+ * @retval SCARD_F_COMM_ERROR Socket is closed.
+ * @retval SCARD_F_COMM_ERROR A signal was received.
  */
-INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
-	uint64_t buffer_size, int32_t filedes, int32_t timeOut)
+INTERNAL LONG MessageReceiveTimeout(uint32_t command, void *buffer_void,
+	uint64_t buffer_size, int32_t filedes, long timeOut)
 {
 	char *buffer = buffer_void;
 
 	/* default is success */
-	int retval = 0;
+	LONG retval = SCARD_S_SUCCESS;
 
 	/* record the time when we started */
 	struct timeval start;
@@ -341,7 +186,7 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 		if (delta > timeOut*1000)
 		{
 			/* we already timed out */
-			retval = -2;
+			retval = SCARD_E_TIMEOUT;
 			break;
 		}
 
@@ -364,7 +209,7 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 			if (!FD_ISSET(filedes, &read_fd))
 			{
 				/* very strange situation. it should be an assert really */
-				retval = -1;
+				retval = SCARD_F_COMM_ERROR;
 				break;
 			}
 			readed = read(filedes, buffer, remaining);
@@ -377,7 +222,7 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 			} else if (readed == 0)
 			{
 				/* peer closed the socket */
-				retval = -1;
+				retval = SCARD_F_COMM_ERROR;
 				break;
 			} else
 			{
@@ -385,24 +230,17 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 				 * other errors are fatal */
 				if (errno != EINTR && errno != EAGAIN)
 				{
-					retval = -1;
+					retval = SCARD_F_COMM_ERROR;
 					break;
 				}
 			}
 		} else if (selret == 0)
 		{
-#ifdef PCSCD
-			(void)command;
-
-			/* timeout */
-			retval = -1;
-			break;
-#else
 			/* is the daemon still there? */
-			if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
+			retval  =  SCardCheckDaemonAvailability();
+			if (retval != SCARD_S_SUCCESS)
 			{
 				/* timeout */
-				retval = -1;
 				break;
 			}
 
@@ -410,7 +248,6 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 			 * this is logged on the client side and not on the pcscd
 			 * side*/
 			Log2(PCSC_LOG_INFO, "Command 0x%X not yet finished", command);
-#endif
 		} else
 		{
 			/* we ignore signals, all other errors are fatal */
@@ -418,7 +255,7 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
 			{
 				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
 					strerror(errno));
-				retval = -1;
+				retval = SCARD_F_COMM_ERROR;
 				break;
 			}
 		}
@@ -437,16 +274,15 @@ INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
  * @param[in] command Command to be sent.
  * @param[in] dwClientID Client socket handle.
  * @param[in] size Size of the message (\p data).
- * @param[in] timeOut Timeout to the operation in ms.
  * @param[in] data_void Data to be sent.
  *
  * @return Same error codes as MessageSend().
  */
-INTERNAL int32_t MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
+INTERNAL LONG MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
 	uint64_t size, void *data_void)
 {
 	struct rxHeader header;
-	int ret;
+	LONG ret;
 
 	/* header */
 	header.command = command;
@@ -454,23 +290,197 @@ INTERNAL int32_t MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
 	ret = MessageSend(&header, sizeof(header), dwClientID);
 
 	/* command */
-	ret = MessageSend(data_void, size, dwClientID);
+	if (size > 0)
+		ret = MessageSend(data_void, size, dwClientID);
 
 	return ret;
 }
 
+#endif
+
+/* functions used by pcscd and libpcsclite */
+
 /**
- * @brief Closes the communications channel used by the server to talk to the
- * clients.
+ * @brief Sends a menssage from client to server or vice-versa.
  *
- * The socket used is closed and the file it is bound to is removed.
+ * Writes the message in the shared file \c filedes.
  *
- * @param[in] sockValue Socket to be closed.
- * @param[in] pcFilePath File used by the socket.
+ * @param[in] buffer_void Message to be sent.
+ * @param[in] buffer_size Size of the message to send
+ * @param[in] filedes Socket handle.
+ *
+ * @retval SCARD_S_SUCCESS Success
+ * @retval SCARD_E_TIMEOUT Timeout.
+ * @retval SCARD_F_COMM_ERROR Socket is closed.
+ * @retval SCARD_F_COMM_ERROR A signal was received.
  */
-INTERNAL void CleanupSharedSegment(int sockValue, const char *pcFilePath)
+INTERNAL LONG MessageSend(void *buffer_void, uint64_t buffer_size,
+	int32_t filedes)
 {
-	(void)close(sockValue);
-	(void)remove(pcFilePath);
+	char *buffer = buffer_void;
+
+	/* default is success */
+	LONG retval = SCARD_S_SUCCESS;
+
+	/* how many bytes remains to be written */
+	size_t remaining = buffer_size;
+
+	/* repeat until all data is written */
+	while (remaining > 0)
+	{
+		fd_set write_fd;
+		int selret;
+
+		FD_ZERO(&write_fd);
+		FD_SET(filedes, &write_fd);
+
+		selret = select(filedes + 1, NULL, &write_fd, NULL, NULL);
+
+		/* try to write only when the file descriptor is writable */
+		if (selret > 0)
+		{
+			int written;
+
+			if (!FD_ISSET(filedes, &write_fd))
+			{
+				/* very strange situation. it should be an assert really */
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			}
+			/* since we are a user library we can't play with signals
+			 * The signals may already be used by the application */
+#ifdef MSG_NOSIGNAL
+			/* Get EPIPE return code instead of SIGPIPE signal
+			 * Works on Linux */
+			written = send(filedes, buffer, remaining, MSG_NOSIGNAL);
+#else
+			/* we may get a SIGPIPE signal if the other side has closed */
+			written = write(filedes, buffer, remaining);
+#endif
+
+			if (written > 0)
+			{
+				/* we wrote something */
+				buffer += written;
+				remaining -= written;
+			} else if (written == 0)
+			{
+				/* peer closed the socket */
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			} else
+			{
+				/* we ignore the signals and socket full situations, all
+				 * other errors are fatal */
+				if (errno != EINTR && errno != EAGAIN)
+				{
+					retval = SCARD_E_NO_SERVICE;
+					break;
+				}
+			}
+		} else if (selret == 0)
+		{
+			/* timeout */
+			retval = SCARD_E_TIMEOUT;
+			break;
+		} else
+		{
+			/* ignore signals */
+			if (errno != EINTR)
+			{
+				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
+					strerror(errno));
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			}
+		}
+	}
+
+	return retval;
+}
+
+/**
+ * @brief Called by the Client to get the reponse from the server or vice-versa.
+ *
+ * Reads the message from the file \c filedes.
+ *
+ * @param[out] buffer_void Message read.
+ * @param[in] buffer_size Size to read
+ * @param[in] filedes Socket handle.
+ *
+ * @retval SCARD_S_SUCCESS Success.
+ * @retval SCARD_F_COMM_ERROR Socket is closed.
+ * @retval SCARD_F_COMM_ERROR A signal was received.
+ */
+INTERNAL LONG MessageReceive(void *buffer_void, uint64_t buffer_size,
+	int32_t filedes)
+{
+	char *buffer = buffer_void;
+
+	/* default is success */
+	LONG retval = SCARD_S_SUCCESS;
+
+	/* how many bytes we must read */
+	size_t remaining = buffer_size;
+
+	/* repeat until we get the whole message */
+	while (remaining > 0)
+	{
+		fd_set read_fd;
+		int selret;
+
+		FD_ZERO(&read_fd);
+		FD_SET(filedes, &read_fd);
+
+		selret = select(filedes + 1, &read_fd, NULL, NULL, NULL);
+
+		/* try to read only when socket is readable */
+		if (selret > 0)
+		{
+			int readed;
+
+			if (!FD_ISSET(filedes, &read_fd))
+			{
+				/* very strange situation. it should be an assert really */
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			}
+			readed = read(filedes, buffer, remaining);
+
+			if (readed > 0)
+			{
+				/* we got something */
+				buffer += readed;
+				remaining -= readed;
+			} else if (readed == 0)
+			{
+				/* peer closed the socket */
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			} else
+			{
+				/* we ignore the signals and empty socket situations, all
+				 * other errors are fatal */
+				if (errno != EINTR && errno != EAGAIN)
+				{
+					retval = SCARD_F_COMM_ERROR;
+					break;
+				}
+			}
+		}
+		else
+		{
+			/* we ignore signals, all other errors are fatal */
+			if (errno != EINTR)
+			{
+				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
+					strerror(errno));
+				retval = SCARD_F_COMM_ERROR;
+				break;
+			}
+		}
+	}
+
+	return retval;
 }
 
