@@ -5,10 +5,10 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.h 4967 2010-05-29 11:28:16Z rousseau $
+ * $Id: winscard_msg.h 5929 2011-09-03 21:43:21Z rousseau $
  */
 
 /**
@@ -25,12 +25,7 @@
 /** Major version of the current message protocol */
 #define PROTOCOL_VERSION_MAJOR 4
 /** Minor version of the current message protocol */
-#define PROTOCOL_VERSION_MINOR 0
-
-#ifdef __cplusplus
-extern "C"
-{
-#endif
+#define PROTOCOL_VERSION_MINOR 2
 
 	/**
 	 * @brief Information transmitted in \ref CMD_VERSION Messages.
@@ -47,7 +42,7 @@ extern "C"
 	 */
 	struct rxHeader
 	{
-		uint32_t size;		/**< size of the message expluding this header */
+		uint32_t size;		/**< size of the message excluding this header */
 		uint32_t command;	/**< one of the \c pcsc_msg_commands */
 	};
 
@@ -68,9 +63,9 @@ extern "C"
 		SCARD_TRANSMIT = 0x09,			/**< used by SCardTransmit() */
 		SCARD_CONTROL = 0x0A,			/**< used by SCardControl() */
 		SCARD_STATUS = 0x0B,			/**< used by SCardStatus() */
-		SCARD_GET_STATUS_CHANGE = 0x0C,	/**< used by SCardGetStatusChange() */
+		SCARD_GET_STATUS_CHANGE = 0x0C,	/**< not used */
 		SCARD_CANCEL = 0x0D,			/**< used by SCardCancel() */
-		SCARD_CANCEL_TRANSACTION = 0x0E,	/**< used by SCardCancelTransaction() */
+		SCARD_CANCEL_TRANSACTION = 0x0E,/**< not used */
 		SCARD_GET_ATTRIB = 0x0F,		/**< used by SCardGetAttrib() */
 		SCARD_SET_ATTRIB = 0x10,		/**< used by SCardSetAttrib() */
 		CMD_VERSION = 0x11,				/**< get the client/server protocol version */
@@ -194,17 +189,6 @@ extern "C"
 		uint32_t rv;
 	};
 
-	/**
-	 * @brief contained in \ref SCARD_CANCEL_TRANSACTION Messages.
-	 *
-	 * These data are passed throw the field \c sharedSegmentMsg.data.
-	 */
-	struct cancel_transaction_struct
-	{
-		int32_t hCard;
-		uint32_t rv;
-	};
-
 	/**
 	 * @brief contained in \ref SCARD_STATUS Messages.
 	 *
@@ -213,12 +197,6 @@ extern "C"
 	struct status_struct
 	{
 		int32_t hCard;
-		char mszReaderNames[MAX_READERNAME];
-		uint32_t pcchReaderLen;
-		uint32_t dwState;
-		uint32_t dwProtocol;
-		uint8_t pbAtr[MAX_ATR_SIZE];
-		uint32_t pcbAtrLen;
 		uint32_t rv;
 	};
 
@@ -272,20 +250,21 @@ extern "C"
 	 * Now some function definitions
 	 */
 
+#ifdef PCSCD
+	int32_t InitializeSocket(void);
+	int32_t ListenExistingSocket(int fd);
+	int32_t ProcessEventsServer(/*@out@*/ uint32_t *);
+#else
+	char *getSocketName(void);
 	int32_t ClientSetupSession(uint32_t *);
 	int32_t ClientCloseSession(uint32_t);
-	int32_t InitializeSocket(void);
-	int32_t ProcessEventsServer(/*@out@*/ uint32_t *);
-	int32_t MessageSend(void *buffer, uint64_t buffer_size, int32_t filedes);
-	int32_t MessageReceive(/*@out@*/ void *buffer, uint64_t buffer_size,
+	LONG MessageReceiveTimeout(uint32_t command, /*@out@*/ void *buffer,
+		uint64_t buffer_size, int32_t filedes, long timeOut);
+	LONG MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
+		uint64_t size, void *data);
+#endif
+	LONG MessageSend(void *buffer, uint64_t buffer_size, int32_t filedes);
+	LONG MessageReceive(/*@out@*/ void *buffer, uint64_t buffer_size,
 		int32_t filedes);
-	int32_t MessageReceiveTimeout(uint32_t command, /*@out@*/ void *buffer,
-		uint64_t buffer_size, int32_t filedes, int32_t timeOut);
-	int32_t MessageSendWithHeader(uint32_t command, uint32_t dwClientID, uint64_t size, void *data);
-	void CleanupSharedSegment(int32_t, const char *);
-
-#ifdef __cplusplus
-}
-#endif
 
 #endif
