@@ -5,12 +5,12 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2010
+ * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  * Copyright (C) 2009
  *  Jean-Luc Giraud <jlgiraud@googlemail.com>
  *
- * $Id: winscard_svc.c 5386 2010-11-08 10:00:58Z rousseau $
+ * $Id: winscard_svc.c 5864 2011-07-09 11:37:48Z rousseau $
  */
 
 /**
@@ -83,8 +83,9 @@ static int contextsListhContext_seeker(const void *el, const void *key)
 
 	if ((el == NULL) || (key == NULL))
 	{
-		Log3(PCSC_LOG_CRITICAL, "called with NULL pointer: el=%X, key=%X",
+		Log3(PCSC_LOG_CRITICAL, "called with NULL pointer: el=%p, key=%p",
 			el, key);
+		return 0;
 	}
 
 	if (currentContext->hContext == *(int32_t *)key)
@@ -284,7 +285,7 @@ static const char *CommandsText[] = {
 #define WRITE_BODY(v) \
 	WRITE_BODY_WITH_COMMAND(CommandsText[header.command], v)
 #define WRITE_BODY_WITH_COMMAND(command, v) \
-	Log4(SCARD_S_SUCCESS == v.rv ? PCSC_LOG_DEBUG : PCSC_LOG_ERROR, "%s rv=0x%X for client %d", command, v.rv, filedes); \
+	Log4(PCSC_LOG_DEBUG, "%s rv=0x%X for client %d", command, v.rv, filedes); \
 	ret = MessageSend(&v, sizeof(v), filedes);
 
 static void ContextThread(LPVOID newContext)
@@ -292,7 +293,7 @@ static void ContextThread(LPVOID newContext)
 	SCONTEXT * threadContext = (SCONTEXT *) newContext;
 	int32_t filedes = threadContext->dwClientID;
 
-	Log3(PCSC_LOG_DEBUG, "Thread is started: dwClientID=%d, threadContext @%X",
+	Log3(PCSC_LOG_DEBUG, "Thread is started: dwClientID=%d, threadContext @%p",
 		threadContext->dwClientID, threadContext);
 
 	while (1)
@@ -351,6 +352,11 @@ static void ContextThread(LPVOID newContext)
 			{
 				/* nothing to read */
 
+#ifdef USE_USB
+				/* wait until all readers are ready */
+				RFWaitForReaderInit();
+#endif
+
 				/* dump the readers state */
 				ret = MessageSend(readerStates, sizeof(readerStates), filedes);
 			}
@@ -828,7 +834,7 @@ static LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard,
 		if (listLength >= contextMaxCardHandles)
 		{
 			Log4(PCSC_LOG_DEBUG,
-				"Too many card handles for thread context @%X: %d (max is %d)"
+				"Too many card handles for thread context @%p: %d (max is %d)"
 				"Restart pcscd with --max-card-handle-per-thread value",
 				threadContext, listLength, contextMaxCardHandles);
 			return SCARD_E_NO_MEMORY;
@@ -908,14 +914,14 @@ static LONG MSGCleanupClient(SCONTEXT * threadContext)
 	}
 
 	Log3(PCSC_LOG_DEBUG,
-		"Thread is stopping: dwClientID=%d, threadContext @%X",
+		"Thread is stopping: dwClientID=%d, threadContext @%p",
 		threadContext->dwClientID, threadContext);
 
 	/* Clear the struct to ensure that we detect
 	 * access to de-allocated memory
 	 * Hopefully the compiler won't optimise it out */
 	memset((void*) threadContext, 0, sizeof(SCONTEXT));
-	Log2(PCSC_LOG_DEBUG, "Freeing SCONTEXT @%X", threadContext);
+	Log2(PCSC_LOG_DEBUG, "Freeing SCONTEXT @%p", threadContext);
 
 	(void)pthread_mutex_lock(&contextsList_lock);
 	lrv = list_delete(&contextsList, threadContext);
